require "passwords_generator/version"
require "passwords_generator/dictionary.rb"

class PasswordGenerator
  VOWELS = [ 'a', 'e', 'i', 'o', 'u', 'y' ]
  CONSOSTANTS = ['b', 'c', 'd', 'f', 'g', 'h', 'j', 'k', 'l', 'm', 'n', 'p', 'q', 'r', 's', 't', 'v', 'w', 'x']
  DEFAULT_GENERATE_OPTIONS = {
    skip_upper_case: false,
    without_word: false,
    skip_number: false,
    length: 8,
    count_upper_case: 2,
    custom_dictionary_file: nil
  }

  def initialize(options={})
    @params = DEFAULT_GENERATE_OPTIONS.merge(options)
    @dictionary = Dictionary.new(@params[:custom_dictionary_file]) unless @params[:without_word]
    @password = ''
  end

  def generate
    @params[:length] < 8 || @params[:without_word] ? generate_without_word : generate_with_word
    unless @params[:skip_upper_case]
      @params[:count_upper_case].times{random_letter_capital}
    end
    @password
  end

  private

    def generate_with_word(length = @params[:length])
      if length > 0 
        const = length >= 6 ? rand(4..6) : length
        @password << random_word(const)
        generate_with_word(length-const)
      else 
        @password
      end
    end

    def generate_without_word 
      @params[:length].times{|number| @password << random_symbol(number) }
    end

    def random_word(length)
      word = @dictionary.words.sample
      if word.length >= length
        word[0..length-1]
      else
        number_length = length - word.length
        number_length.times{ |i| word << (@params[:skip_number] ? random_symbol(i) : rand(10).to_s)}
        word
      end
    end

    def random_symbol(number)
      number.even? ? VOWELS.sample : CONSOSTANTS.sample
    end

    def random_letter_capital
      letter = capital_letter
      if letter[1].nil? 
        random_letter_capital
      else 
        @password[letter[0]] = letter[1].upcase
      end
    end

    def capital_letter 
      random_index = rand(@password.length - 1)
      random_letter = @password[random_index]
      letter = /[a-z]/.match(random_letter).nil? ? nil : random_letter
      [random_index, letter]
    end 
end